/////////////////////////////////////////////////////////////////////////////////////////////////////////
// SimpleGamePlugin
// MyBehavior.cpp
// 
// Code : Anael Seghezzi
/////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "MyBehavior.h"


/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Init, this part is always similar, constructor, copy constructor etc
/////////////////////////////////////////////////////////////////////////////////////////////////////////

// constructor
MyBehavior::MyBehavior(MObject3d * parentObject):
MBehavior(parentObject)
{
}

// copy constructor
MyBehavior::MyBehavior(MyBehavior & behavior, MObject3d * parentObject):
MBehavior(parentObject)
{}

// destructor
MyBehavior::~MyBehavior(void)
{}

// destroy function : always similar
void MyBehavior::destroy(void)
{
	delete this;
}

// getNew function : always similar
MBehavior * MyBehavior::getNew(MObject3d * parentObject)
{
	return new MyBehavior(parentObject);
}

// getCopy function : always similar
MBehavior * MyBehavior::getCopy(MObject3d * parentObject)
{
	return new MyBehavior(*this, parentObject);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Variables, allow to access custom variable from script and from Maratis Editor
/////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int MyBehavior::getVariablesNumber(void){
	return 2;
}

MVariable MyBehavior::getVariable(unsigned int id)
{
	switch(id)
	{
	default:
		return MVariable("NULL", NULL, M_VARIABLE_NULL);
	case 0:
		return MVariable("boneName", &m_boneName, M_VARIABLE_STRING);
	case 1:
		return MVariable("targetName", &m_targetName, M_VARIABLE_STRING);
	}
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions
/////////////////////////////////////////////////////////////////////////////////////////////////////////

void MyBehavior::copyTransformation(MOEntity * entity, MOBone * bone, MObject3d * object)
{
	// ignore the scale
	MMatrix4x4 objectNoScaleMatrix;
	objectNoScaleMatrix.setScale(object->getTransformedScale());
	objectNoScaleMatrix = (*object->getMatrix()) * objectNoScaleMatrix.getInverse();
	
	// target matrix
	MMatrix4x4 targetMatrix = entity->getMatrix()->getInverse() * objectNoScaleMatrix;
	*bone->getMatrix() = targetMatrix; // this is the actual armature space matrix

    MObject3d * parentBone = bone->getParent();
    if(parentBone)
    {
        // calculate target matrix in parentBone's space
        targetMatrix = parentBone->getMatrix()->getInverse() * targetMatrix;
    }

    // get position and rotation coords from targetMatrix
    bone->setPosition(targetMatrix.getTranslationPart());
    bone->setEulerRotation(targetMatrix.getEulerAngles());
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Events
/////////////////////////////////////////////////////////////////////////////////////////////////////////

// update function (called by MGame class by default)
void MyBehavior::update(void)
{
	MEngine * engine = MEngine::getInstance();
	MLevel * level = engine->getLevel();
	
	MScene * scene = level->getCurrentScene();
	
	MOEntity * parent = (MOEntity *)getParentObject();
	MOEntity * target = scene->getEntityByName(m_targetName.getSafeString());
	
	if(target)
	{
		MArmature * armature = parent->getMesh()->getArmature();
		if(armature)
		{
			MOBone * bone = armature->getBoneByName(m_boneName.getSafeString());
			if(bone)
				copyTransformation(parent, bone, target);
		}
	}
}
